'use strict';

var fs = require('fs');
var path = require('path');
var shelljs = require('shelljs');
var matched = require('matched');
var htmlImportsVisitor = require('html-imports-visitor');
var parse5 = require('parse5');
var dom5 = require('dom5');

var entry = "\0rollup-plugin-html-entry:entry-point";

var index = function (config) {
  var include = "**/*.html";
  var exclude = [];
  var exporter = function (path$$1) { return ("export * from " + (JSON.stringify(path$$1)) + ";"); };

  var configure = function (options) {
    if (typeof options === "string") {
      include = [options];
    } else if (Array.isArray(options)) {
      include = options;
    } else {
      include = options.include || [];
      exclude = options.exclude || [];
      if (options.exports === false) {
        exporter = function (path$$1) { return ("import " + (JSON.stringify(path$$1)) + ";"); };
      }
    }
    if (exclude.length) {
      exclude = matched.sync(exclude, { realpath: true });
    }
  };

  if (config) {
    configure(config);
  }

  var realPaths = {};
  var virtualPaths = {};
  var pathsList = [];
  var htmls = {};
  var blackList = {};

  var visitor = {

    import: function import$1(link, ref) {
      var location = ref.location;

      if (blackList[location]) {
        dom5.remove(link);
      }
    },

    enter: function enter(document, ref) {
      var path$$1 = ref.path;

      htmls[path$$1] = document;
    },

    visit: function visit(element, ref) {
      var path$$1 = ref.path;
      var index = ref.index;

      realPaths[path$$1] = true;
      var src = dom5.getAttribute(element, "src");
      if (src) {
        if (htmlImportsVisitor.isLocal(src)) {
          pathsList.push(htmlImportsVisitor.resolve(path$$1, src));
          dom5.remove(element);
        }
      } else {
        // (todo) should avoid possibile collision against real paths
        var virtualPath = path$$1 + "_" + index + ".js";
        pathsList.push(virtualPath);
        virtualPaths[virtualPath] = dom5.getTextContent(element);
        dom5.remove(element);
      }
    }

  };

  var analyze = function () {
    for (var i in exclude) {
      blackList[exclude[i]] = true;
    }
    var helper = new htmlImportsVisitor.VisitorHelper(visitor, htmlImportsVisitor.predicates.hasTagName("script"));
    for (var i$1 in include) {
      helper.enter(include[i$1]);
    }
  };

  var output = function (destPath) {
    var promises = [];
    var loop = function ( filepath ) {
      var doc = htmls[filepath];
      // (todo) check the correctness of path construction
      var destFilepath = path.resolve(destPath, path.relative("./", filepath));
      promises.push(new Promise(function (res, rej) { // eslint-disable-line no-loop-func
        shelljs.mkdir("-p", path.dirname(destFilepath));
        fs.writeFile(destFilepath, parse5.serialize(doc), function (err) {
          if (err) {
            rej(err);
          } else {
            res();
          }
        });
      }));
    };

    for (var filepath in htmls) loop( filepath );
    return Promise.all(promises)
  };

  return {
    options: function options(options$1) {
      if (options$1.entry && options$1.entry !== entry) {
        configure(options$1.entry);
      }
      options$1.entry = entry;
      analyze();
    },

    resolveId: function resolveId(id) {
      if (id === entry) {
        return entry
      }

      if (id in virtualPaths) {
        return id
      }
    },

    load: function load(id) {
      if (id === entry) {
        if (!pathsList.length) {
          return Promise.resolve("")
        }
        return new Promise(
          function (res) { return res(pathsList.map(exporter).join("\n")); }
        )
      }

      if (id in virtualPaths) {
        return virtualPaths[id]
      }
    },

    ongenerate: function ongenerate() {
      if (config.output) {
        return output(config.output)
      }
    }
  }

};

module.exports = index;
