import { writeFile } from 'fs';
import { dirname, relative, resolve } from 'path';
import { mkdir } from 'shelljs';
import { sync } from 'matched';
import { VisitorHelper, isLocal, predicates, resolve as resolve$1 } from 'html-imports-visitor';
import { serialize } from 'parse5';
import { getAttribute, getTextContent, remove } from 'dom5';

var entry = "\0rollup-plugin-html-entry:entry-point";

var index = function (config) {
  var include = "**/*.html";
  var exclude = [];
  var exporter = function (path$$1) { return ("export * from " + (JSON.stringify(path$$1)) + ";"); };

  var configure = function (options) {
    if (typeof options === "string") {
      include = [options];
    } else if (Array.isArray(options)) {
      include = options;
    } else {
      include = options.include || [];
      exclude = options.exclude || [];
      if (options.exports === false) {
        exporter = function (path$$1) { return ("import " + (JSON.stringify(path$$1)) + ";"); };
      }
    }
    if (exclude.length) {
      exclude = sync(exclude, { realpath: true });
    }
  };

  if (config) {
    configure(config);
  }

  var realPaths = {};
  var virtualPaths = {};
  var pathsList = [];
  var htmls = {};
  var blackList = {};

  var visitor = {

    import: function import$1(link, ref) {
      var location = ref.location;

      if (blackList[location]) {
        remove(link);
      }
    },

    enter: function enter(document, ref) {
      var path$$1 = ref.path;

      htmls[path$$1] = document;
    },

    visit: function visit(element, ref) {
      var path$$1 = ref.path;
      var index = ref.index;

      realPaths[path$$1] = true;
      var src = getAttribute(element, "src");
      if (src) {
        if (isLocal(src)) {
          pathsList.push(resolve$1(path$$1, src));
          remove(element);
        }
      } else {
        // (todo) should avoid possibile collision against real paths
        var virtualPath = path$$1 + "_" + index + ".js";
        pathsList.push(virtualPath);
        virtualPaths[virtualPath] = getTextContent(element);
        remove(element);
      }
    }

  };

  var analyze = function () {
    for (var i in exclude) {
      blackList[exclude[i]] = true;
    }
    var helper = new VisitorHelper(visitor, predicates.hasTagName("script"));
    for (var i$1 in include) {
      helper.enter(include[i$1]);
    }
  };

  var output = function (destPath) {
    var promises = [];
    var loop = function ( filepath ) {
      var doc = htmls[filepath];
      // (todo) check the correctness of path construction
      var destFilepath = resolve(destPath, relative("./", filepath));
      promises.push(new Promise(function (res, rej) { // eslint-disable-line no-loop-func
        mkdir("-p", dirname(destFilepath));
        writeFile(destFilepath, serialize(doc), function (err) {
          if (err) {
            rej(err);
          } else {
            res();
          }
        });
      }));
    };

    for (var filepath in htmls) loop( filepath );
    return Promise.all(promises)
  };

  return {
    options: function options(options$1) {
      if (options$1.entry && options$1.entry !== entry) {
        configure(options$1.entry);
      }
      options$1.entry = entry;
      analyze();
    },

    resolveId: function resolveId(id) {
      if (id === entry) {
        return entry
      }

      if (id in virtualPaths) {
        return id
      }
    },

    load: function load(id) {
      if (id === entry) {
        if (!pathsList.length) {
          return Promise.resolve("")
        }
        return new Promise(
          function (res) { return res(pathsList.map(exporter).join("\n")); }
        )
      }

      if (id in virtualPaths) {
        return virtualPaths[id]
      }
    },

    ongenerate: function ongenerate() {
      if (config.output) {
        return output(config.output)
      }
    }
  }

};

export default index;
